pub struct NoLearningResolver;Expand description
ConflictResolver which does not attempt to learn from conflicts.
The idea is to backtrack a single level and try the negated [Predicate] of the previous
decision.