pub struct ResolutionResolver { /* private fields */ }Expand description
ConflictResolver which resolves conflicts according to the CDCL procedure.
This conflict resolver will derive a nogood that is implied by the constraints already present in the solver. This new nogood is added as a constraint to the solver, and the solver backtracks to the decision level at which the new constraint propagates.
The ResolutionResolver can be used in two AnalysisModes:
AnalysisMode::OneUIP- Resolves until finding the unit implication point.- AnalysisMode::AllDecision - Resolves until the learned nogood contains only decisions.
For an in-depth explanation and overview of CDCL and UIP, see [1].
Bibliography
[1] J. Marques-Silva, I. Lynce, and S. Malik, ‘Conflict-driven clause learning SAT solvers’, Handbook of satisfiability, pp. 131–153, 2009.
Implementations
Sourceimpl ResolutionResolver
impl ResolutionResolver
pub fn new(mode: AnalysisMode, should_minimise: bool) -> ResolutionResolver