ConflictAnalysisContext

Struct ConflictAnalysisContext 

pub struct ConflictAnalysisContext<'a> { /* private fields */ }
Used during conflict analysis to provide the necessary information.

All fields are made public for the time being for simplicity. In the future that may change.

Implementations

Source

impl ConflictAnalysisContext<'_>

pub fn get_state(&self) -> &State

pub fn post(&mut self, predicate: Predicate) -> Result<bool, EmptyDomain>

Apply a Predicate to the State.

Returns true if a change to a domain occured, and false if the given Predicate was already true.

If a domain becomes empty due to this operation, an EmptyDomain error is returned.

pub fn restore_to(&mut self, checkpoint: usize)

Restore to the given checkpoint.

If the provided checkpoint is equal to the current checkpoint, this is a no-op. If the provided checkpoint is larger than the current checkpoint, this method will panic.

See State::new_checkpoint for an example.

pub fn get_conflict_nogood(&mut self) -> Vec<Predicate>

Returns a nogood which led to the conflict, excluding predicates from the root decision level.

pub fn get_propagation_reason( &mut self, predicate: Predicate, current_nogood: CurrentNogood<'_>, reason_buffer: &mut (impl Extend<Predicate> + AsRef<[Predicate]>), )

Compute the reason for predicate being true. The reason will be stored in reason_buffer.

If predicate is not true, or it is a decision, then this function will panic.

pub fn find_last_decision(&mut self) -> Option<Predicate>

Returns the last decision which was made by the solver (if such a decision exists).

Source

impl ConflictAnalysisContext<'_>

Methods used for proof logging

pub fn explain_root_assignment(&mut self, predicate: Predicate)

Explains the root assignment of predicate in the proof log.

pub fn log_deduction( &mut self, premises: impl IntoIterator<Item = Predicate>, ) -> ConstraintTag

Log a deduction (learned nogood) to the proof.

The inferences and marked propagations are assumed to be recorded in reverse-application order.

Source

impl ConflictAnalysisContext<'_>

Methods used for keeping track of statistics.

pub fn predicate_appeared_in_conflict(&mut self, predicate: Predicate)

Informs the used Brancher that the provided predicate appeared during conflict analysis.

This is used by Branchers such as [AutonomousSearch] to guide the search.

Source

impl ConflictAnalysisContext<'_>

pub fn process_learned_nogood( &mut self, learned_nogood_predicates: Vec<Predicate>, lbd: u32, ) -> usize

Backtracks the solver and adds the learned nogood to the database, returning the level to which the solver backtracked.