Module predicates

Module predicates 

Contains structures which represent certain predicates.

The solver only utilizes the following types of predicates:

  • A predicate of the form [x >= v]
  • A predicate of the form [x <= v]
  • A predicate of the form [x = v]
  • A predicate of the form [x != v]

In general, these Predicates are used to represent propagations, explanations or decisions.

Structs

Lbd
Used to compute the LBD of nogoods. The type carries state that prevents the re-allocation of helper data structures.
Predicate
Representation of a domain operation, also known as an atomic constraint. It is a triple (DomainId, PredicateType, value).
PredicateIdGenerator
PropositionalConjunction
A struct which represents a conjunction of Predicates (e.g. it can represent [x >= 5] /\ [y <= 10]).

Enums

PredicateType

Traits

PredicateConstructor
A trait which defines methods for creating a Predicate.