pub struct Predicate { /* private fields */ }Expand description
Representation of a domain operation, also known as an atomic constraint. It is a triple
(DomainId, PredicateType, value).
To create a Predicate, use Predicate::new or the more concise predicate! macro.
Order
Predicates have a well-defined order. They are first ordered by the domain, and then by
predicate type, and finally by the value. The order is chosen such that for a fixed domain x,
predicates are ordered as follows:
[>= 5], [>= 7], [!= 2], [!= 3], [== 5], [!= 7], [<= 6], [<= 10]
From the order, we get the lower-bound predicates first, ordered by non-decreasing bound, then the (not-)equal predicates, ordered by non-decreasing bound, then the upper-bound predicates, ordered by non-increasing bounds.
Implementations
Sourceimpl Predicate
impl Predicate
Sourcepub fn new(id: DomainId, predicate_type: PredicateType, value: i32) -> Predicate
pub fn new(id: DomainId, predicate_type: PredicateType, value: i32) -> Predicate
Creates a new Predicate (also known as atomic constraint) which represents a domain
operation.
Sourcepub fn implies(&self, other: Predicate) -> bool
pub fn implies(&self, other: Predicate) -> bool
Returns true if self implies other.
Example
let x = DomainId::new(0);
assert!(predicate![x >= 5].implies(predicate![x >= 3]));
assert!(predicate![x >= 5].implies(predicate![x != 1]));
assert!(predicate![x == 5].implies(predicate![x <= 5]));pub fn get_predicate_type(&self) -> PredicateType
Sourceimpl Predicate
impl Predicate
pub fn is_equality_predicate(&self) -> bool
pub fn is_lower_bound_predicate(&self) -> bool
pub fn is_upper_bound_predicate(&self) -> bool
pub fn is_not_equal_predicate(&self) -> bool
Sourcepub fn get_domain(&self) -> DomainId
pub fn get_domain(&self) -> DomainId
pub fn get_right_hand_side(&self) -> i32
pub fn trivially_true() -> Predicate
pub fn trivially_false() -> Predicate