#[repr(u8)]pub enum Priority {
High = 0,
Medium = 1,
Low = 2,
VeryLow = 3,
}Expand description
The priority of a propagator, used for determining the order in which propagators will be called.
Propagators with high priority are propagated before propagators with low(er) priority. If two propagators have the same priority, then the order in which they are propagated is unspecified.
Typically, propagators with low computational complexity should be assigned a high priority (i.e., should be propagated before computationally expensive propagators).
Variants