pub struct EventsToRegister(/* private fields */);Expand description
Contains all the events and domains that a propagator needs to be enqueued for.
Implementations
Sourceimpl EventsToRegister
impl EventsToRegister
Sourcepub fn empty() -> EventsToRegister
pub fn empty() -> EventsToRegister
Create an EventsToRegister without any variables.
This is the uncommon case. Without registering for variable events, a propagator will never be enqueued automatically. However, certain propagators like, e.g., compound propagators, may not be able to register during construction in which case they will be enqueued explicitly when a constraint is added to them. The nogood propagator is an example of such a propagator.
Sourcepub fn builder() -> EventsToRegisterBuilder
pub fn builder() -> EventsToRegisterBuilder
Create a new EventsToRegisterBuilder.
If no event registrations will be made, use EventsToRegister::empty instead.
Calling EventsToRegisterBuilder::build without any registrations will cause a panic.
Sourcepub fn add(
&mut self,
target: &impl EventTarget,
domain_events: DomainEvents,
local_id: LocalId,
)
pub fn add( &mut self, target: &impl EventTarget, domain_events: DomainEvents, local_id: LocalId, )
Add a new event registration to an existing instance of self.
Example
use pumpkin_core::propagation::DomainEvents;
use pumpkin_core::propagation::EventsToRegister;
use pumpkin_core::propagation::LocalId;
use pumpkin_core::variables::DomainId;
let v1 = DomainId::new(0);
let v2 = DomainId::new(0);
let mut registration = EventsToRegister::builder()
.add(&v1, DomainEvents::ANY_INT, LocalId::from(0))
.build();
// Extend the events to register with another variable.
registration.add(&v2, DomainEvents::ANY_INT, LocalId::from(1));Sourcepub fn iter(&self) -> impl ExactSizeIterator
pub fn iter(&self) -> impl ExactSizeIterator
Iterate the registrations already made.