ExplanationContext

Struct ExplanationContext 

pub struct ExplanationContext<'a> { /* private fields */ }
The context that is available when lazily explaining propagations.

See [Propagator] for more information.

Implementations

Source

impl<'a> ExplanationContext<'a>

pub fn get_predicate(&mut self, predicate_id: PredicateId) -> Predicate

pub fn working_nogood(&self) -> impl Iterator<Item = Predicate>

Get the current working nogood.

The working nogood does not necessarily contain the predicate that is being explained. However, the explanation will be used to either resolve with the working nogood or minimize it some other way.

pub fn get_trail_position(&self) -> usize

Returns the trail position before which the propagation took place.

For example, if the context is created for explaining a predicate [x >= v] at trail position i, then this method will return i - 1