pub struct LocalId(/* private fields */);Expand description
A local id uniquely identifies a variable within a specific propagator. A local id can be thought of as the index of the variable in the propagator.
Implementations