pub struct PropagationContext<'a> { /* private fields */ }Expand description
Provides information about the state of the solver to a propagator.
Domains can be read through the implementation of [ReadDomains], and changes to the state can
be made via Self::post.
Implementations
Sourceimpl<'a> PropagationContext<'a>
impl<'a> PropagationContext<'a>
Sourcepub fn register_predicate(&mut self, predicate: Predicate) -> PredicateId
pub fn register_predicate(&mut self, predicate: Predicate) -> PredicateId
Register the propagator to be enqueued when the provided Predicate becomes true.
Returns the PredicateId assigned to the provided predicate, which will be provided
to [Propagator::notify_predicate_id_satisfied].
Sourcepub fn unregister_predicate(&mut self, predicate_id: PredicateId)
pub fn unregister_predicate(&mut self, predicate_id: PredicateId)
Stop being enqueued for the given predicate.
Sourcepub fn register_domain_event(
&mut self,
var: impl IntegerVariable,
domain_events: DomainEvents,
local_id: LocalId,
)
pub fn register_domain_event( &mut self, var: impl IntegerVariable, domain_events: DomainEvents, local_id: LocalId, )
Subscribes the propagator to the given DomainEvents.
See [PropagatorConstructorContext::register] for more information.
Sourcepub fn unregister_domain_event(
&mut self,
var: impl IntegerVariable,
local_id: LocalId,
)
pub fn unregister_domain_event( &mut self, var: impl IntegerVariable, local_id: LocalId, )
Stop being enqueued for events on the given integer variable.
Sourcepub fn get_predicate(&mut self, predicate_id: PredicateId) -> Predicate
pub fn get_predicate(&mut self, predicate_id: PredicateId) -> Predicate
Get the Predicate for a given PredicateId.
Sourcepub fn get_id(&mut self, predicate: Predicate) -> PredicateId
pub fn get_id(&mut self, predicate: Predicate) -> PredicateId
Get a PredicateId for the given Predicate.
If no ID exists, one will be created.
pub fn reborrow(&mut self) -> PropagationContext<'_>
Sourceimpl PropagationContext<'_>
impl PropagationContext<'_>
Sourcepub fn post(
&mut self,
predicate: Predicate,
reason: impl Into<Reason>,
inference_code: &InferenceCode,
) -> Result<(), EmptyDomainConflict>
pub fn post( &mut self, predicate: Predicate, reason: impl Into<Reason>, inference_code: &InferenceCode, ) -> Result<(), EmptyDomainConflict>
Assign the truth-value of the given Predicate to true in the current partial
assignment.
If the truth-value is already true, then this is a no-op. Alternatively, if the
truth-value is false, then a conflict is triggered and the EmptyDomain error is
returned. At that point, no-more propagation should happen.