pub struct PropagatorConstructorContext<'a> { /* private fields */ }Expand description
PropagatorConstructorContext is used when Propagators are initialised after creation.
It represents a communication point between the [Solver] and the Propagator.
Propagators use the PropagatorConstructorContext to register to domain changes
of variables and to retrieve the current bounds of variables.
Implementations
Sourceimpl PropagatorConstructorContext<'_>
impl PropagatorConstructorContext<'_>
Sourcepub fn will_not_register_any_events(&mut self)
pub fn will_not_register_any_events(&mut self)
Indicate that the constructor is deliberately not registering the propagator to be enqueued at any time.
If this is called and later a registration happens, then the registration will still go through. Calling this function only prevents the crash if no registration happens.
Sourcepub fn register(
&mut self,
var: impl IntegerVariable,
domain_events: DomainEvents,
local_id: LocalId,
)
pub fn register( &mut self, var: impl IntegerVariable, domain_events: DomainEvents, local_id: LocalId, )
Subscribes the propagator to the given DomainEvents.
The domain events determine when Propagator::notify() will be called on the propagator.
The LocalId is internal information related to the propagator,
which is used when calling Propagator::notify() to identify the variable.
Each variable must have a unique LocalId. Most often this would be its index of the
variable in the internal array of variables.
Duplicate registrations are ignored.
Sourcepub fn register_predicate(&mut self, predicate: Predicate) -> PredicateId
pub fn register_predicate(&mut self, predicate: Predicate) -> PredicateId
Register the propagator to be enqueued when the given Predicate becomes true.
Returns the PredicateId used by the solver to track the predicate.
Sourcepub fn register_backtrack<Var>(
&mut self,
var: Var,
domain_events: DomainEvents,
local_id: LocalId,
)where
Var: IntegerVariable,
pub fn register_backtrack<Var>(
&mut self,
var: Var,
domain_events: DomainEvents,
local_id: LocalId,
)where
Var: IntegerVariable,
Subscribes the propagator to the given DomainEvents when they are undone during
backtracking. This method is complementary to PropagatorConstructorContext::register,
the LocalIds provided to both of these method should be the same for the same variable.
The domain events determine when Propagator::notify_backtrack() will be called on the
propagator. The LocalId is internal information related to the propagator,
which is used when calling Propagator::notify_backtrack() to identify the variable.
Each variable must have a unique LocalId. Most often this would be its index of the
variable in the internal array of variables.
Note that the LocalId is used to differentiate between [DomainId]s and
[AffineView]s.
Sourcepub fn reborrow(&mut self) -> PropagatorConstructorContext<'_>
pub fn reborrow(&mut self) -> PropagatorConstructorContext<'_>
Reborrow the current context to a new value with a shorter lifetime. Should be used when
passing Self to another function that takes ownership, but the value is still needed
afterwards.
Sourcepub fn add_inference_checker(
&mut self,
inference_code: InferenceCode,
checker: Box<dyn InferenceChecker<Predicate>>,
)
pub fn add_inference_checker( &mut self, inference_code: InferenceCode, checker: Box<dyn InferenceChecker<Predicate>>, )
Add an inference checker for inferences produced by the propagator.
If the check-propagations feature is not enabled, adding an InferenceChecker will not
do anything.