pub struct RuntimeCheckers { /* private fields */ }Expand description
Holds the runtime checkers that are added by a propagator.
Used when creating a new propagator in [PropagatorConstructor::create].
Implementations
Sourceimpl RuntimeCheckers
impl RuntimeCheckers
Sourcepub fn empty() -> RuntimeCheckers
pub fn empty() -> RuntimeCheckers
Create a RuntimeCheckers value which we accept may be empty.
This is often not what you want. If it is expected that some checkers should be added,
use RuntimeCheckers::builder instead to communicate that intention.
Sourcepub fn builder() -> RuntimeCheckersBuilder
pub fn builder() -> RuntimeCheckersBuilder
Create a RuntimeCheckersBuilder to add runtime checkers.
The RuntimeCheckersBuilder::build will panic if no checkers are added.
Sourcepub fn add_inference_checker(
&mut self,
constraint_tag: ConstraintTag,
inference_label: impl InferenceLabel,
checker: impl InferenceChecker<Predicate> + 'static,
) -> InferenceCode
pub fn add_inference_checker( &mut self, constraint_tag: ConstraintTag, inference_label: impl InferenceLabel, checker: impl InferenceChecker<Predicate> + 'static, ) -> InferenceCode
Add an InferenceChecker to verify the soundness of propagations.