Modules
Structs
- Reified
Checker - Reified
Propagator - Propagator for the constraint
r -> p, whereris a Boolean literal andpis an arbitrary propagator. - Reified
Propagator Args - A
PropagatorConstructorfor the reified propagator.
r -> p, where r is a Boolean literal and p is an arbitrary
propagator.PropagatorConstructor for the reified propagator.