ReifiedPropagator

Struct ReifiedPropagator 

pub struct ReifiedPropagator<WrappedPropagator> { /* private fields */ }
Propagator for the constraint r -> p, where r is a Boolean literal and p is an arbitrary propagator.

When a propagator is reified, it will only propagate whenever r is set to true. However, if the propagator implements Propagator::detect_inconsistency, the result of that method may be used to propagate r to false. If that method is not implemented, r will never be propagated to false.