pub enum OptimisationResult<Stop> {
Optimal(Solution),
Satisfiable(Solution),
Stopped(Solution, Stop),
Unsatisfiable,
Unknown,
}Expand description
The result of a call to Solver::optimise.
Variants
Optimal(Solution)
Indicates that an optimal solution has been found and proven to be optimal. It provides an
instance of Solution which contains the optimal solution.
Satisfiable(Solution)
Indicates that a solution was found and provides an instance of Solution which contains
best known solution by the solver.
Stopped(Solution, Stop)
The optimisation was stopped by the solution callback.
Unsatisfiable
Indicates that there is no solution to the problem.
Unknown
Indicates that it is not known whether a solution exists. This is likely due to a
[TerminationCondition] triggering.