pub enum Conflict {
Propagator(PropagatorConflict),
EmptyDomain(EmptyDomainConflict),
}Expand description
Information concerning the conflict returned by State::propagate_to_fixed_point.
Two (related) conflicts can happen:
- a propagator explicitly detects a conflict.
- a propagator post a domain change that results in a variable having an empty domain.
Variants
Propagator(PropagatorConflict)
A conflict raised explicitly by a propagator.
EmptyDomain(EmptyDomainConflict)
A conflict caused by an empty domain for a variable occurring.