pub struct PredicateHeap { /* private fields */ }Expand description
A max-heap of predicates. The keys are based on the trail positions of the predicates in the state, meaning predicates are popped in reverse trail order. Implied predicates are popped before the predicate on the trail that implies the predicate.
Implementations
Sourceimpl PredicateHeap
impl PredicateHeap
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
See BinaryHeap::is_empty.
Sourcepub fn pop(&mut self) -> Option<Predicate>
pub fn pop(&mut self) -> Option<Predicate>
See BinaryHeap::pop.
Sourcepub fn push(&mut self, predicate: Predicate, state: &State)
pub fn push(&mut self, predicate: Predicate, state: &State)
Push a new predicate onto the heap.
Its priority will be based on its trail position in the given state. This heap will
return elements through Self::pop by reverse-trail order.
If the predicate is not true in the given state, this method panics.