PropagatorConflict

Struct PropagatorConflict 

pub struct PropagatorConflict {
    pub conjunction: PropositionalConjunction,
    pub inference_code: InferenceCode,
}
A conflict stated by a propagator. A propagator that identifies a conflict that is not an empty domain, describes that conflict with this type.

Fields

conjunction: PropositionalConjunction

The conjunction that describes the infeasible partial assignment.

inference_code: InferenceCode

The inference code that identified the conflict.