pub struct TimeTableOverIntervalPropagator<Var> { /* private fields */ }Expand description
Propagator responsible for using time-table reasoning to propagate the Cumulative constraint
where a time-table is a structure which stores the mandatory resource usage of the tasks at
different time-points - This method creates a resource profile over an interval rather than
creating one per time-point (as is done in [TimeTablePerPointPropagator]).
See Sections 4.2.1, 4.5.2 and 4.6.1-4.6.3 of [1] for more information about time-table reasoning.
[1] A. Schutt, Improving scheduling by learning. University of Melbourne, Department of Computer Science and Software Engineering, 2011.
Implementations
Sourceimpl<Var> TimeTableOverIntervalPropagator<Var>where
Var: IntegerVariable + 'static,
impl<Var> TimeTableOverIntervalPropagator<Var>where
Var: IntegerVariable + 'static,
pub fn new( arg_tasks: &[ArgTask<Var>], capacity: i32, cumulative_options: CumulativePropagatorOptions, constraint_tag: ConstraintTag, ) -> TimeTableOverIntervalPropagator<Var>