Module element

Module element 

Contains the propagator for the Element constraint.

Structs

ElementArgs
ElementChecker
ElementPropagator
Arc-consistent propagator for constraint element([x_1, \ldots, x_n], i, e), where x_j are variables, i is an integer variable, and e is a variable, which holds iff x_i = e