pub struct CheckerStore { /* private fields */ }Expand description
Owns the runtime checkers present in the solver.
The runtime checkers consist of:
- inference checkers, which verify that propagations are sound.
Implementations
Sourceimpl CheckerStore
impl CheckerStore
Sourcepub fn for_inference_code(
&self,
inference_code: &InferenceCode,
) -> impl ExactSizeIterator
pub fn for_inference_code( &self, inference_code: &InferenceCode, ) -> impl ExactSizeIterator
Get the [InferenceChecker]s for the given inference code.
Sourcepub fn add_inference_checker(
&mut self,
inference_code: InferenceCode,
checker: BoxedChecker<Predicate>,
)
pub fn add_inference_checker( &mut self, inference_code: InferenceCode, checker: BoxedChecker<Predicate>, )
Add a new inference checker for the inference code.
An inference code can have multiple checkers, so if an inference checker was already registered for the given code, this new checker is simply added to the collection.