pub struct ConflictAnalysisContext<'a> { /* private fields */ }Expand description
Used during conflict analysis to provide the necessary information.
All fields are made public for the time being for simplicity. In the future that may change.
Implementations
Sourceimpl ConflictAnalysisContext<'_>
impl ConflictAnalysisContext<'_>
pub fn get_state(&self) -> &State
Sourcepub fn post(&mut self, predicate: Predicate) -> Result<bool, EmptyDomain>
pub fn post(&mut self, predicate: Predicate) -> Result<bool, EmptyDomain>
Apply a Predicate to the State.
Returns true if a change to a domain occured, and false if the given Predicate was
already true.
If a domain becomes empty due to this operation, an EmptyDomain error is returned.
Sourcepub fn restore_to(&mut self, checkpoint: usize)
pub fn restore_to(&mut self, checkpoint: usize)
Restore to the given checkpoint.
If the provided checkpoint is equal to the current checkpoint, this is a no-op. If the provided checkpoint is larger than the current checkpoint, this method will panic.
See State::new_checkpoint for an example.
Sourcepub fn get_conflict_nogood(&mut self) -> Vec<Predicate>
pub fn get_conflict_nogood(&mut self) -> Vec<Predicate>
Returns a nogood which led to the conflict, excluding predicates from the root decision level.
Sourcepub fn get_propagation_reason(
&mut self,
predicate: Predicate,
current_nogood: CurrentNogood<'_>,
reason_buffer: &mut (impl Extend<Predicate> + AsRef<[Predicate]>),
)
pub fn get_propagation_reason( &mut self, predicate: Predicate, current_nogood: CurrentNogood<'_>, reason_buffer: &mut (impl Extend<Predicate> + AsRef<[Predicate]>), )
Compute the reason for predicate being true. The reason will be stored in
reason_buffer.
If predicate is not true, or it is a decision, then this function will panic.
Sourcepub fn find_last_decision(&mut self) -> Option<Predicate>
pub fn find_last_decision(&mut self) -> Option<Predicate>
Returns the last decision which was made by the solver (if such a decision exists).
Sourceimpl ConflictAnalysisContext<'_>
Methods used for proof logging
impl ConflictAnalysisContext<'_>
Methods used for proof logging
Sourcepub fn explain_root_assignment(&mut self, predicate: Predicate)
pub fn explain_root_assignment(&mut self, predicate: Predicate)
Explains the root assignment of predicate in the proof log.
Sourcepub fn log_deduction(
&mut self,
premises: impl IntoIterator<Item = Predicate>,
) -> ConstraintTag
pub fn log_deduction( &mut self, premises: impl IntoIterator<Item = Predicate>, ) -> ConstraintTag
Log a deduction (learned nogood) to the proof.
The inferences and marked propagations are assumed to be recorded in reverse-application order.
Sourceimpl ConflictAnalysisContext<'_>
Methods used for keeping track of statistics.
impl ConflictAnalysisContext<'_>
Methods used for keeping track of statistics.
Sourcepub fn predicate_appeared_in_conflict(&mut self, predicate: Predicate)
pub fn predicate_appeared_in_conflict(&mut self, predicate: Predicate)
Sourceimpl ConflictAnalysisContext<'_>
impl ConflictAnalysisContext<'_>