SmallRng

Struct SmallRng 

pub struct SmallRng(/* private fields */);
A small-state, fast, non-crypto, non-portable PRNG

This is the “standard small” RNG, a generator with the following properties:

  • Non-portable: any future library version may replace the algorithm and results may be platform-dependent. (For a small portable generator, use the rand_pcg or rand_xoshiro crate.)
  • Non-cryptographic: output is easy to predict (insecure)
  • Quality: statistically good quality
  • Fast: the RNG is fast for both bulk generation and single values, with consistent cost of method calls
  • Fast initialization
  • Small state: little memory usage (current state size is 16-32 bytes depending on platform)

The current algorithm is Xoshiro256PlusPlus on 64-bit platforms and Xoshiro128PlusPlus on 32-bit platforms. Both are also implemented by the rand_xoshiro crate.

Seeding (construction)

This generator implements the SeedableRng trait. All methods are suitable for seeding, but note that, even with a fixed seed, output is not portable. Some suggestions:

  1. To automatically seed with a unique seed, use rand::make_rng():
    use rand::rngs::SmallRng;
    let mut rng: SmallRng = rand::make_rng();
  2. To use a deterministic integral seed, use seed_from_u64. This uses a hash function internally to yield a (typically) good seed from any input.
    let rng = SmallRng::seed_from_u64(1);
  3. To seed deterministically from text or other input, use rand_seeder.

See also Seeding RNGs in the book.

Generation

The generator implements Rng and thus also RngExt. See also the Random Values chapter in the book.