Contains a number of propagators for a variety of arithmetic constraints.
Structs
- Absolute
Value Args - Absolute
Value Checker - Absolute
Value Propagator - Propagator for
absolute = |signed|, whereabsoluteandsignedare integer variables. - Binary
Equals Checker - Binary
Equals Propagator - Propagator for the constraint
a = b. - Binary
Equals Propagator Args - The
PropagatorConstructorfor theBinaryEqualsPropagator. - Binary
NotEquals Checker - Binary
NotEquals Propagator - Propagator for the constraint
a != b. - Binary
NotEquals Propagator Args - The
PropagatorConstructorfor theBinaryNotEqualsPropagator. - Division
Args - The
PropagatorConstructorfor theDivisionPropagator. - Division
Propagator - A propagator for maintaining the constraint
numerator / denominator = rhs; note that this propagator performs truncating division (i.e. rounding towards 0). - Integer
Division Checker - Integer
Multiplication Args - The
PropagatorConstructorforIntegerMultiplicationPropagator. - Integer
Multiplication Checker - Integer
Multiplication Propagator - A propagator for maintaining the constraint
a * b = c. The propagator (currently) only propagates the signs of the variables, the case where a, b, c >= 0, and detects a conflict if the variables are fixed. - Linear
Less OrEqual Inference Checker - Linear
Less OrEqual Propagator - Propagator for the constraint
\sum x_i <= c. - Linear
Less OrEqual Propagator Args - The
PropagatorConstructorfor theLinearLessOrEqualPropagator. - Linear
NotEqual Checker - Linear
NotEqual Propagator - Propagator for the constraint
\sum x_i != rhs, wherex_iare integer variables andrhsis an integer constant. - Linear
NotEqual Propagator Args - The
PropagatorConstructorfor theLinearNotEqualPropagator. - Maximum
Args - Maximum
Checker - Maximum
Propagator - Bounds-consistent propagator which enforces
max(array) = rhs. Can be constructed throughMaximumArgs.