Module arithmetic

Module arithmetic 

Contains a number of propagators for a variety of arithmetic constraints.

Structs

AbsoluteValueArgs
AbsoluteValueChecker
AbsoluteValuePropagator
Propagator for absolute = |signed|, where absolute and signed are integer variables.
BinaryEqualsChecker
BinaryEqualsPropagator
Propagator for the constraint a = b.
BinaryEqualsPropagatorArgs
The PropagatorConstructor for the BinaryEqualsPropagator.
BinaryNotEqualsChecker
BinaryNotEqualsPropagator
Propagator for the constraint a != b.
BinaryNotEqualsPropagatorArgs
The PropagatorConstructor for the BinaryNotEqualsPropagator.
DivisionArgs
The PropagatorConstructor for the DivisionPropagator.
DivisionPropagator
A propagator for maintaining the constraint numerator / denominator = rhs; note that this propagator performs truncating division (i.e. rounding towards 0).
IntegerDivisionChecker
IntegerMultiplicationChecker
Verifies that a claimed inference for a * b = c is actually implied by its premises.
IntegerMultiplicationConstructor
The PropagatorConstructor for IntegerMultiplicationPropagator.
IntegerMultiplicationPropagator
A bounds(R)-consistent propagator for maintaining the constraint a * b = c.
LinearLessOrEqualInferenceChecker
LinearLessOrEqualPropagator
Propagator for the constraint \sum x_i <= c.
LinearLessOrEqualPropagatorArgs
The PropagatorConstructor for the LinearLessOrEqualPropagator.
LinearNotEqualChecker
LinearNotEqualPropagator
Propagator for the constraint \sum x_i != rhs, where x_i are integer variables and rhs is an integer constant.
LinearNotEqualPropagatorArgs
The PropagatorConstructor for the LinearNotEqualPropagator.
MaximumArgs
MaximumChecker
MaximumPropagator
Bounds-consistent propagator which enforces max(array) = rhs. Can be constructed through MaximumArgs.