pub struct TimeTableOverIntervalIncrementalPropagator<Var, const SYNCHRONISE: bool> { /* private fields */ }Expand description
Propagator responsible for using time-table reasoning to propagate the Cumulative constraint
where a time-table is a structure which stores the mandatory resource usage of the tasks at
different time-points - This method creates a resource profile over an interval rather than
creating one per time-point (hence the name). Furthermore, the
[TimeTableOverIntervalPropagator] has a generic argument which represents the type of variable
used for modelling the start variables, this will be an implementation of IntegerVariable.
The difference between the TimeTableOverIntervalIncrementalPropagator and
[TimeTableOverIntervalPropagator] is that the TimeTableOverIntervalIncrementalPropagator
does not recalculate the time-table from scratch whenever the
Propagator::propagate method is called but it utilises the
Propagator::notify method to determine when a mandatory part is added
and only updates the structure based on these updated mandatory parts.
See Sections 4.2.1, 4.5.2 and 4.6.1-4.6.3 of [1] for more information about time-table reasoning.
[1] A. Schutt, Improving scheduling by learning. University of Melbourne, Department of Computer Science and Software Engineering, 2011.
Implementations
Sourceimpl<Var, const SYNCHRONISE: bool> TimeTableOverIntervalIncrementalPropagator<Var, SYNCHRONISE>where
Var: IntegerVariable + 'static,
impl<Var, const SYNCHRONISE: bool> TimeTableOverIntervalIncrementalPropagator<Var, SYNCHRONISE>where
Var: IntegerVariable + 'static,
pub fn new( arg_tasks: &[ArgTask<Var>], capacity: i32, cumulative_options: CumulativePropagatorOptions, constraint_tag: ConstraintTag, ) -> TimeTableOverIntervalIncrementalPropagator<Var, SYNCHRONISE>